<?php

	class BuckarooPayment
	{
		private $sWebsiteKey = false;
		private $sSecretKey = false;
		private $sPaymentMethod = false;
		private $sLanguageCode = false;

		private $sReturnUrl = false;
		private $sReportUrl = false;

		private $sCachePath;
		private $bTestMode = false;
		private $sIdealIssuer = false;

		private $sOrderId = false;
		private $fAmount = false;
		private $sDescription = false;
		private $sCurrencyCode = 'EUR';


		private $aCurrencies = array(
			'creditcard' => array(
				'ARS', 'AUD', 'BRL', 'CAD', 'CHF', 'CNY', 'CZK', 'DKK', 'EUR', 'GBP', 'HRK', 'ISK', 'JPY', 'LTL', 'LVL', 'MXN', 'NOK', 'NZD', 'PLN', 'RUB', 'SEK', 'TRY', 'USD', 'ZAR'
			),
			'paypal' => array(
				'AUD', 'BRL', 'CAD', 'CHF', 'DKK', 'EUR', 'GBP', 'HKD', 'HUF', 'ILS', 'JPY', 'MYR', 'NOK', 'NZD', 'PHP', 'PLN', 'SEK', 'SGD', 'THB', 'TRL', 'TWD', 'USD'
			),
			'transfer' => array(
				'EUR', 'GBP', 'PLN'
			),
			'p24' => array(
				'PLN'
			)
		);

		private $aTransaction = false;

		public function __construct($sWebsiteKey, $sSecretKey)
		{
			$this->sWebsiteKey = $sWebsiteKey;
			$this->sSecretKey = $sSecretKey;
		}

		// Should point to directory where cache is stored (Use FALSE for NO CACHE), make sure directory has read/write privileges
		public function setCachePath($sPath = false)
		{
			$this->sCachePath = $sPath;
		}

		public function setCurrencyCode($sCurrencyCode = false)
		{
			if(is_bool($sCurrencyCode))
			{
				$this->sCurrencyCode = false;
				return true;
			}
			elseif(is_string($sCurrencyCode))
			{
				$sCurrencyCode = strtoupper(substr($sCurrencyCode, 0, 3));

				if(!empty($this->sPaymentMethod))
				{
					if(isset($this->aCurrencies[$this->sPaymentMethod]))
					{
						if(in_array($sCurrencyCode, $this->aCurrencies[$this->sPaymentMethod]))
						{
							$this->sCurrencyCode = $sCurrencyCode;
							return true;
						}
					}
				}
			}

			return false;
		}

		public function setOrder($sOrderId, $fAmount, $sDescription = false)
		{
			$this->sOrderId = $sOrderId;
			$this->fAmount = $fAmount;
			$this->sDescription = $sDescription;

			if(empty($this->sDescription))
			{
				$this->sDescription = 'Webshop bestelling ' . $this->sOrderId;
			}

			return true;
		}

		public function setPaymentMethod($sPaymentMethod = false)
		{
			if(is_bool($sPaymentMethod))
			{
				$this->sPaymentMethod = false;
				return true;
			}
			elseif(is_string($sPaymentMethod))
			{
				$sPaymentMethod = strtolower($sPaymentMethod);

				//, 'maestro', 'mastercard', 'paypal', 'visa', 'v_pay', 'bancontact'

				if(in_array($sPaymentMethod, array('ideal')))
				{
					$this->sPaymentMethod = $sPaymentMethod;
					return true;
				}
			}

			return false;
		}

		public function setTestmode($bEnabled = true)
		{
			return ($this->bTestMode = $bEnabled);
		}

		public function setIdealIssuer($sIdealIssuer)
		{
			$this->sIdealIssuer = $sIdealIssuer;
			return true;
		}

		public function setReturnUrl($sReturnUrl = false)
		{
			$this->sReturnUrl = $sReturnUrl;
			return true;
		}

		public function setReportUrl($sReportUrl = false)
		{
			$this->sReportUrl = $sReportUrl;
			return true;
		}

		public function getIssuerList()
		{
			// Since Buckaroo doesnt have an endpoint to retreive the list, we shall build an array ourselfs.

			// Do request
			$sJson = '';

			if(!empty($sJson))
			{
				$aIssuers = json_decode($sJson, true);
			}
			else
			{
				$aIssuers = array(
					'ABNANL2A' => 'ABN AMRO',
					'ASNBNL21' => 'ASN Bank',
					'BUNQNL2A' => 'bunq',
					'HANDNL2A' => 'Handelsbanken',
					'INGBNL2A' => 'ING Bank',
					'KNABNL2H' => 'Knab',
					'MOYONL21' => 'Moneyou',
					'RABONL2U' => 'Rabobank',
					'RBRBNL21' => 'RegioBank',
					'SNSBNL2A' => 'SNS Bank',
					'TRIONL2U' => 'Triodos Bank',
					'FVLBNL22' => 'van Lanschot'
				);
			}

			return $aIssuers;
		}


		public function getTransaction()
		{
			if(empty($this->sWebsiteKey))
			{
				idealcheckout_log('No Website Key found.', __FILE__, __LINE__);

				$this->aTransaction = array('error' => array('message' => 'No Website Key found.'));
				return false;
			}
			elseif(empty($this->sSecretKey))
			{
				idealcheckout_log('No Secret Key found.', __FILE__, __LINE__);

				$this->aTransaction = array('error' => array('message' => 'No Secret Key found.'));
				return false;
			}
			elseif(empty($this->sOrderId))
			{
				idealcheckout_log('No order ID found.', __FILE__, __LINE__);

				$this->aTransaction = array('error' => array('message' => 'No order ID found.'));
				return false;
			}
			elseif(empty($this->fAmount))
			{
				idealcheckout_log('No amount found.', __FILE__, __LINE__);

				$this->aTransaction = array('error' => array('message' => 'No amount found.'));
				return false;
			}
			elseif(empty($this->sReturnUrl))
			{
				idealcheckout_log('No return URL found.', __FILE__, __LINE__);

				$this->aTransaction = array('error' => array('message' => 'No return URL found.'));
				return false;
			}
			elseif(empty($this->sReportUrl))
			{
				idealcheckout_log('No report URL found.', __FILE__, __LINE__);

				$this->aTransaction = array('error' => array('message' => 'No report URL found.'));
				return false;
			}


			$aRequest = array(
				'Currency' => $this->sCurrencyCode,
				'AmountDebit' => (float)$this->fAmount,
				'Invoice' => $this->sOrderId,
				'ReturnURL' => $this->sReturnUrl,
				'PushURL' => $this->sReportUrl,
				'Services' => array(
					'ServiceList' => array(
						array(
							'Action' => 'Pay',
							'Name' => $this->sPaymentMethod,
							'Parameters' => array(
								array(
									'Name' => 'issuer',
									'Value' => $this->sIdealIssuer
								)
							)
						)
					)
				)
			);

			$sApiUrl = 'https://' . ($this->bTestMode ? 'test' : '') . 'checkout.buckaroo.nl/json/transaction';
			$sPostData = json_encode($aRequest, JSON_PRETTY_PRINT);

			$sRequestUrl = strtolower(rawurlencode(($this->bTestMode ? 'test' : '') . 'checkout.buckaroo.nl/json/transaction'));

			$sAuthorization = $this->createAuthorization($sPostData, $sRequestUrl);

			$sResponse = idealcheckout_doHttpRequest($sApiUrl, $sPostData, true, 30, false, array('Authorization: hmac ' . $sAuthorization));

			if(!empty($sResponse))
			{
				$this->aTransaction = json_decode($sResponse, true);

				if($this->aTransaction)
				{
					if((isset($this->aTransaction['Key'], $this->aTransaction['RequiredAction'], $this->aTransaction['RequiredAction']['RedirectURL'])))
					{
						return true;
					}
					elseif(isset($this->aTransaction['RequestErrors'], $this->aTransaction['RequestErrors']['ParameterErrors'], $this->aTransaction['RequestErrors']['ParameterErrors']['Error'], $this->aTransaction['RequestErrors']['ParameterErrors']['ErrorMessage']))
					{
						$this->aTransaction = array('error' => array('Message' => $this->aTransaction['RequestErrors']['ParameterErrors']['Error'] . ':' . $this->aTransaction['RequestErrors']['ParameterErrors']['ErrorMessage']));
					}
				}
			}
			else
			{
				$this->aTransaction = array('error' => array('message' => 'No response received from Buckaroo (See logs).'));
			}

			idealcheckout_log($aRequest, __FILE__, __LINE__);
			idealcheckout_log($sResponse, __FILE__, __LINE__);

			return false;
		}

		public function getTransactionId()
		{
			if(!empty($this->aTransaction['Key']))
			{
				return $this->aTransaction['Key'];
			}

			return false;
		}

		public function getTransactionUrl()
		{
			if(!empty($this->aTransaction['RequiredAction']['RedirectURL']))
			{
				return $this->aTransaction['RequiredAction']['RedirectURL'];
			}

			return false;
		}

		public function getError()
		{
			if(!empty($this->aTransaction['RequestErrors']['ParameterErrors']['ErrorMessage']))
			{
				return $this->aTransaction['RequestErrors']['ParameterErrors']['ErrorMessage'];
			}

			return false;
		}

		public function getStatus($sTransactionId)
		{
			$sApiUrl = 'https://' . ($this->bTestMode ? 'test' : '') . 'checkout.buckaroo.nl/json/transaction/status/' . $sTransactionId;

			$sRequestUrl = strtolower(rawurlencode(($this->bTestMode ? 'test' : '') . 'checkout.buckaroo.nl/json/transaction/status/' . $sTransactionId));

			$sAuthorization = $this->createAuthorization('', $sRequestUrl);

			$sResponse = idealcheckout_doHttpRequest($sApiUrl, '', true, 30, true, array('Authorization: hmac ' . $sAuthorization));

			if(!empty($sResponse))
			{
				$aResponse = json_decode($sResponse, true);

				if(isset($aResponse['Status'], $aResponse['Status']['Code'], $aResponse['Status']['Code']['Code']))
				{
					$iResult = $aResponse['Status']['Code']['Code'];

					return $this->doTranslateStatus($iResult);
				}
			}

			return '';
		}
		
		private function doTranslateStatus($iResult)
		{
			if(!empty($iResult))
			{
				if($iResult === 190)
				{
					return 'SUCCESS';
				}
				elseif(in_array($iResult, array(790, 791, 792)))
				{
					return 'PENDING';
				}
				elseif(in_array($iResult, array(490, 491, 492, 690)))
				{
					return 'FAILURE';
				}
				elseif(in_array($iResult, array(890, 891)))
				{
					return 'CANCELLED';
				}
				else
				{
					return 'EXPIRED';
				}
			}
			
			return '';
		}

		private function createAuthorization($sPostData, $sRequestUrl)
		{
			if(!empty($sPostData))
			{
				$sRequestMethod = 'POST';
				$sMd5 = md5($sPostData, true);
				$sRequestContentData = base64_encode($sMd5);
			}
			else
			{
				$sRequestMethod = 'GET';
				$sRequestContentData = '';
			}

			$sRequestTimestamp = time();
			$sNonce = strtolower(idealcheckout_getRandomCode(32));

			$sHashString = $this->sWebsiteKey . $sRequestMethod . $sRequestUrl . $sRequestTimestamp . $sNonce . $sRequestContentData;

			$sHash = hash_hmac('sha256', $sHashString, $this->sSecretKey, true);
			$sHmac = base64_encode($sHash);

			// Create authentication header
			return $this->sWebsiteKey . ':' . $sHmac . ':' . $sNonce . ':' . $sRequestTimestamp;
		}

		public function getStatusFromPost($aData = array())
		{
			if(!empty($aData))
			{
				$sPostSignature = $aData['brq_signature'];
				unset($aData['brq_signature']);

				$sHashString = '';

				foreach($aData as $k => $v)
				{
					$sValue = $this->decodePushValue($k, $v);
					$sHashString .= $k. '=' . $sValue;
				}

				$sHashString .= SECRET_KEY;

				$sSignature = sha1($sHashString);

				if((strcmp($sSignature, $sPostSignature) === 0) && !empty($aData['brq_statuscode']))
				{
					return $this->doTranslateStatus($aData['brq_statuscode']);
				}
			}
			
			return false;
		}

		private function decodePushValue($sBuckarooKey, $sValue)
		{
			$aKeys = array();
			$aKeys[] = 'brq_customer_name';
			$aKeys[] = 'brq_SERVICE_ideal_consumerName';
			$aKeys[] = 'brq_SERVICE_transfer_consumerName';
			$aKeys[] = 'brq_SERVICE_payconiq_PayconiqAndroidUrl';
			$aKeys[] = 'brq_SERVICE_paypal_payerEmail';
			$aKeys[] = 'brq_SERVICE_paypal_payerFirstname';
			$aKeys[] = 'brq_SERVICE_paypal_payerLastname';
			$aKeys[] = 'brq_SERVICE_payconiq_PayconiqIosUrl';
			$aKeys[] = 'brq_SERVICE_payconiq_PayconiqUrl';
			$aKeys[] = 'brq_SERVICE_payconiq_QrUrl';
			$aKeys[] = 'brq_SERVICE_masterpass_CustomerPhoneNumber';
			$aKeys[] = 'brq_SERVICE_masterpass_ShippingRecipientPhoneNumber';
			$aKeys[] = 'brq_InvoiceDate';
			$aKeys[] = 'brq_DueDate';
			$aKeys[] = 'brq_PreviousStepDateTime';
			$aKeys[] = 'brq_EventDateTime';

			if(in_array($sBuckarooKey, $aKeys))
			{
				$sDecodedValue = $sValue;
			}
			else
			{
				$sDecodedValue = urldecode($sValue);
			}

			return $sDecodedValue;
		}
	}

?>